# import unittest

# from analogai.foundation.infrastructure.config import ENVIRONMENT

# class EnvironmentCheckMixin:
#     def setUp(self):
#         super().setUp()
#         if ENVIRONMENT == 'production':
#             self.skipTest(
#                 "CRITICAL: Tests cannot be run in production environment! "
#                 "Set ENVIRONMENT=staging or ENVIRONMENT=development to run tests."
#             )

# def prevent_tests_in_production():
#     if ENVIRONMENT == 'production':
#         raise RuntimeError(
#             "CRITICAL: Tests cannot be run in production environment! "
#             "Set ENVIRONMENT=staging or ENVIRONMENT=development to run tests."
#         )

# prevent_tests_in_production()

